@@ -104,10 +104,10 @@ def mean_pressure_weighted(pressure, *args, height=None, bottom=None, depth=None
     Parameters
     ----------
     pressure : `pint.Quantity`
-        Atmospheric pressure profile
+        Atmospheric pressure profile.
 
     args : `pint.Quantity`
-        Parameters for which the pressure-weighted mean is to be calculated
+        Parameters for which the weighted-continuous mean is to be calculated.
 
     height : `pint.Quantity`, optional
         Heights from sounding. Standard atmosphere heights assumed (if needed)
@@ -120,12 +120,25 @@ def mean_pressure_weighted(pressure, *args, height=None, bottom=None, depth=None
         assumed to be the surface.
 
     depth: `pint.Quantity`, optional
-        Depth of the layer in meters or hPa
+        Depth of the layer in meters or hPa.
 
     Returns
     -------
     list of `pint.Quantity`
-        list of layer mean value for each profile in args
+        list of layer mean value for each profile in args.
+
+    See Also
+    --------
+    weighted_continuous_average
+
+    Examples
+    --------
+    >>> from metpy.calc import mean_pressure_weighted
+    >>> from metpy.units import units
+    >>> p = [1000, 850, 700, 500] * units.hPa
+    >>> T = [30, 15, 5, -5] * units.degC
+    >>> mean_pressure_weighted(p, T)
+    [<Quantity(298.54368, 'kelvin')>]
 
     Notes
     -----
@@ -148,6 +161,65 @@ def mean_pressure_weighted(pressure, *args, height=None, bottom=None, depth=None
     return [np.trapz(var_prof * pres_prof, x=pres_prof) / pres_int for var_prof in others]
 
 
+@exporter.export
+@preprocess_and_wrap()
+@check_units('[pressure]')
+def weighted_continuous_average(pressure, *args, height=None, bottom=None, depth=None):
+    r"""Calculate weighted-continuous mean of an arbitrary variable through a layer.
+
+    Layer top and bottom specified in height or pressure.
+
+    Formula based on that from [Holton2004]_ pg. 76 and the NCL function _wgt_vertical_n_
+
+    .. math::  WCA = \frac{\int_{p_s}^{p} A dp}{\int_{p_s}^{p} dp},
+
+    * :math:`WCA` is the weighted continuous average of a variable.
+    * :math:`p` is the bottom pressure level.
+    * :math:`p_s` is the top pressure level.
+    * :math:`A` is the variable whose weighted continuous average is being calculated.
+
+    Parameters
+    ----------
+    pressure : `pint.Quantity`
+        Atmospheric pressure profile.
+
+    args : `pint.Quantity`
+        Parameters for which the weighted-continuous mean is to be calculated.
+
+    height : `pint.Quantity`, optional
+        Heights from sounding. Standard atmosphere heights assumed (if needed)
+        if no heights are given.
+
+    bottom: `pint.Quantity`, optional
+        The bottom of the layer in either the provided height coordinate
+        or in pressure. Don't provide in meters AGL unless the provided
+        height coordinate is meters AGL. Default is the first observation,
+        assumed to be the surface.
+
+    depth: `pint.Quantity`, optional
+        Depth of the layer in meters or hPa.
+
+    Returns
+    -------
+    list of `pint.Quantity`
+        list of layer mean value for each profile in args.
+
+    Notes
+    -----
+    Only functions on 1D profiles (not higher-dimension vertical cross sections or grids).
+    Since this function returns scalar values when given a profile, this will return Pint
+    Quantities even when given xarray DataArray profiles.
+
+    """
+    # Split pressure profile from other variables to average
+    pres_prof, *others = get_layer(
+        pressure, *args, height=height, bottom=bottom, depth=depth
+    )
+
+    return [np.trapz(var_prof, x=pres_prof) / (pres_prof[-1] - pres_prof[0])
+            for var_prof in others]
+
+
 @exporter.export
 @preprocess_and_wrap()
 @check_units('[pressure]', '[speed]', '[speed]', '[length]')
@@ -191,6 +263,20 @@ def bunkers_storm_motion(pressure, u, v, height):
     wind_mean: (`pint.Quantity`, `pint.Quantity`)
         Scalar U- and V- components of surface to 6 km mean flow
 
+    Examples
+    --------
+    >>> from metpy.calc import bunkers_storm_motion, wind_components
+    >>> from metpy.units import units
+    >>> p = [1000, 925, 850, 700, 500, 400] * units.hPa
+    >>> h = [250, 700, 1500, 3100, 5720, 7120] * units.meters
+    >>> wdir = [165, 180, 190, 210, 220, 250] * units.degree
+    >>> sped = [5, 15, 20, 30, 50, 60] * units.knots
+    >>> u, v = wind_components(sped, wdir)
+    >>> bunkers_storm_motion(p, u, v, h)
+    (<Quantity([22.73539654 10.27331352], 'knot')>,
+    <Quantity([ 7.27954821 34.99751866], 'knot')>,
+    <Quantity([15.00747237 22.63541609], 'knot')>)
+
     Notes
     -----
     Only functions on 1D profiles (not higher-dimension vertical cross sections or grids).
@@ -273,6 +359,17 @@ def bulk_shear(pressure, u, v, height=None, bottom=None, depth=None):
     v_shr: `pint.Quantity`
         V-component of layer bulk shear
 
+    Examples
+    --------
+    >>> from metpy.calc import bulk_shear, wind_components
+    >>> from metpy.units import units
+    >>> p = [1000, 925, 850, 700, 500] * units.hPa
+    >>> wdir = [165, 180, 190, 210, 220] * units.degree
+    >>> sped = [5, 15, 20, 30, 50] * units.knots
+    >>> u, v = wind_components(sped, wdir)
+    >>> bulk_shear(p, u, v)
+    (<Quantity(2.41943319, 'knot')>, <Quantity(11.6920573, 'knot')>)
+
     Notes
     -----
     Only functions on 1D profiles (not higher-dimension vertical cross sections or grids).
@@ -326,6 +423,14 @@ def supercell_composite(mucape, effective_storm_helicity, effective_shear):
     `pint.Quantity`
         Supercell composite
 
+    Examples
+    --------
+    >>> from metpy.calc import supercell_composite
+    >>> from metpy.units import units
+    >>> supercell_composite(2500 * units('J/kg'), 125 * units('m^2/s^2'),
+    ...                     50 * units.knot).to_base_units()
+    <Quantity([6.25], 'dimensionless')>
+
     """
     effective_shear = np.clip(np.atleast_1d(effective_shear), None, units.Quantity(20, 'm/s'))
     effective_shear[effective_shear < units.Quantity(10, 'm/s')] = units.Quantity(0, 'm/s')
@@ -377,6 +482,14 @@ def significant_tornado(sbcape, surface_based_lcl_height, storm_helicity_1km, sh
     `pint.Quantity`
         Significant tornado parameter
 
+    Examples
+    --------
+    >>> from metpy.calc import significant_tornado
+    >>> from metpy.units import units
+    >>> significant_tornado(3000 * units('J/kg'), 750 * units.meters,
+    ...                     150 * units('m^2/s^2'), 25 * units.knot).to_base_units()
+    <Quantity([1.28611111], 'dimensionless')>
+
     """
     surface_based_lcl_height = np.clip(np.atleast_1d(surface_based_lcl_height),
                                        units.Quantity(1000., 'm'), units.Quantity(2000., 'm'))
@@ -430,6 +543,18 @@ def critical_angle(pressure, u, v, height, u_storm, v_storm):
     `pint.Quantity`
         Critical angle in degrees
 
+    Examples
+    --------
+    >>> from metpy.calc import critical_angle, wind_components
+    >>> from metpy.units import units
+    >>> p = [1000, 925, 850, 700, 500, 400] * units.hPa
+    >>> h = [250, 700, 1500, 3100, 5720, 7120] * units.meters
+    >>> wdir = [165, 180, 190, 210, 220, 250] * units.degree
+    >>> sped = [5, 15, 20, 30, 50, 60] * units.knots
+    >>> u, v = wind_components(sped, wdir)
+    >>> critical_angle(p, u, v, h, 7 * units.knots, 7 * units.knots)
+    <Quantity(67.0942521, 'degree')>
+
     Notes
     -----
     Only functions on 1D profiles (not higher-dimension vertical cross sections or grids).
