@@ -689,7 +689,8 @@ class MapPanel(Panel, ValidationMixin):
 
     layers = List(Union([Unicode(), Instance('cartopy.feature.Feature')]),
                   default_value=['coastline'])
-    layers.__doc__ = """A list of strings for a pre-defined feature layer or a Cartopy Feature object.
+    layers.__doc__ = """A list of strings for a pre-defined feature layer or a Cartopy Feature
+    object.
 
     Like the projection, there are a couple of pre-defined feature layers that can be called
     using a short name. The pre-defined layers are: 'coastline', 'states', 'borders', 'lakes',
@@ -697,15 +698,45 @@ class MapPanel(Panel, ValidationMixin):
     Cartopy Feature objects.
     """
 
+    layers_edgecolor = List(Unicode(allow_none=True), default_value=['black'])
+    layers_edgecolor.__doc__ = """A list of strings for a pre-defined edgecolor for a layer.
+
+    An option to set a different color for the map layer edge colors. Length of list should
+    match that of layers if not using default value. Behavior is to repeat colors if not enough
+    provided by user. Use `None` value for 'ocean', 'lakes', 'rivers', and 'land'.
+    """
+
+    layers_linewidth = List(Union([Int(), Float()], allow_none=True), default_value=[1])
+    layers_linewidth.__doc__ = """A list of values defining the linewidth for a layer.
+
+    An option to set a different color for the map layer edge colors. Length of list should
+    match that of layers if not using default value. Behavior is to repeat colors if not enough
+    provided by user. Use `None` value for 'ocean', 'lakes', 'rivers', and 'land'.
+    """
+
     title = Unicode()
     title.__doc__ = """A string to set a title for the figure.
 
     This trait sets a user-defined title that will plot at the top center of the figure.
     """
 
+    left_title = Unicode(allow_none=True, default_value=None)
+    left_title.__doc__ = """A string to set a title for the figure with the location on the
+    top left of the figure.
+
+    This trait sets a user-defined title that will plot at the top left of the figure.
+    """
+
+    right_title = Unicode(allow_none=True, default_value=None)
+    right_title.__doc__ = """A string to set a title for the figure with the location on the
+    top right of the figure.
+
+    This trait sets a user-defined title that will plot at the top right of the figure.
+    """
+
     title_fontsize = Union([Int(), Float(), Unicode()], allow_none=True, default_value=None)
-    title_fontsize.__doc__ = """An integer or string value for the font size of the title of the
-    figure.
+    title_fontsize.__doc__ = """An integer or string value for the font size of the title of
+    the figure.
 
     This trait sets the font size for the title that will plot at the top center of the figure.
     Accepts size in points or relative size. Allowed relative sizes are those of Matplotlib:
@@ -866,12 +897,31 @@ class MapPanel(Panel, ValidationMixin):
                     p.draw()
 
             # Add all of the maps
-            for feat in self._layer_features:
-                self.ax.add_feature(feat)
+            if len(self.layers) > len(self.layers_edgecolor):
+                self.layers_edgecolor *= len(self.layers)
+            if len(self.layers) > len(self.layers_linewidth):
+                self.layers_linewidth *= len(self.layers)
+            for i, feat in enumerate(self._layer_features):
+                if self.layers[i] in ['', 'land', 'lake', 'river']:
+                    color = 'face'
+                else:
+                    color = self.layers_edgecolor[i]
+                width = self.layers_linewidth[i]
+                self.ax.add_feature(feat, edgecolor=color, linewidth=width)
 
             # Use the set title or generate one.
-            title = self.title or ',\n'.join(plot.name for plot in self.plots)
-            self.ax.set_title(title, fontsize=self.title_fontsize)
+            if (self.right_title is None) and (self.left_title is None):
+                title = self.title or ',\n'.join(plot.name for plot in self.plots)
+                self.ax.set_title(title, fontsize=self.title_fontsize)
+            else:
+                if self.title is not None:
+                    self.ax.set_title(self.title, fontsize=self.title_fontsize)
+                if self.right_title is not None:
+                    self.ax.set_title(self.right_title, fontsize=self.title_fontsize,
+                                      loc='right')
+                if self.left_title is not None:
+                    self.ax.set_title(self.left_title, fontsize=self.title_fontsize,
+                                      loc='left')
             self._need_redraw = False
 
     def __copy__(self):
@@ -1234,7 +1284,8 @@ class ContourTraits(MetPyHasTraits):
     """
 
     label_fontsize = Union([Int(), Float(), Unicode()], allow_none=True, default_value=None)
-    label_fontsize.__doc__ = """An integer, float, or string value to set the font size of labels for contours.
+    label_fontsize.__doc__ = """An integer, float, or string value to set the font size of
+    labels for contours.
 
     This trait sets the font size for labels that will plot along contour lines. Accepts
     size in points or relative size. Allowed relative sizes are those of Matplotlib:
@@ -1261,7 +1312,8 @@ class ColorfillTraits(MetPyHasTraits):
     """
 
     colorbar = Unicode(default_value=None, allow_none=True)
-    colorbar.__doc__ = """A string (horizontal/vertical) on whether to add a colorbar to the plot.
+    colorbar.__doc__ = """A string (horizontal/vertical) on whether to add a colorbar to the
+    plot.
 
     To add a colorbar associated with the plot, set the trait to ``horizontal`` or
     ``vertical``,specifying the orientation of the produced colorbar. The default value is
@@ -1353,12 +1405,14 @@ class ContourPlot(PlotScalar, ContourTraits, ValidationMixin):
     line.
     """
 
-    linestyle = Unicode('solid', allow_none=True)
-    linestyle.__doc__ = """A string value to set the linestyle (e.g., dashed); default is
-    solid.
+    linestyle = Unicode(None, allow_none=True)
+    linestyle.__doc__ = """A string value to set the linestyle (e.g., dashed), or `None`;
+    default is `None`, which, when using monochrome line colors, uses solid lines for positive
+    values and dashed lines for negative values.
 
-    The valid string values are those of Matplotlib which are solid, dashed, dotted, and
-    dashdot.
+    The valid string values are those of Matplotlib which are 'solid', 'dashed', 'dotted', and
+    'dashdot', as well as their short codes ('-', '--', '.', '-.'). The object `None`, as
+    described above, can also be used.
     """
 
     @observe('contours', 'linecolor', 'linewidth', 'linestyle', 'clabels', 'label_fontsize')
@@ -1404,6 +1458,28 @@ class FilledContourPlot(PlotScalar, ColorfillTraits, ContourTraits, ValidationMi
                                               **kwargs)
 
 
+@exporter.export
+class RasterPlot(PlotScalar, ColorfillTraits):
+    """Make raster plots by defining relevant traits."""
+
+    @observe('image_range', 'colorbar', 'colormap')
+    def _set_need_rebuild(self, _):
+        """Handle changes to attributes that need to regenerate everything."""
+        # Because matplotlib doesn't let you just change these properties, we need
+        # to trigger a clear and re-call of pcolormesh()
+        self.clear()
+
+    def _build(self):
+        """Build the raster plot by calling any plotting methods as necessary."""
+        x_like, y_like, imdata = self.plotdata
+
+        kwargs = plot_kwargs(imdata)
+
+        self.handle = self.parent.ax.pcolormesh(x_like, y_like, imdata,
+                                                cmap=self._cmap_obj, norm=self._norm_obj,
+                                                **kwargs)
+
+
 @exporter.export
 class PlotVector(Plots2D):
     """Defines common elements for 2D vector plots.
@@ -1587,7 +1663,77 @@ class BarbPlot(PlotVector, ValidationMixin):
         self.handle = self.parent.ax.barbs(
             x_like[wind_slice], y_like[wind_slice],
             u.values[wind_slice], v.values[wind_slice],
-            color=self.color, pivot=self.pivot, length=self.barblength, **kwargs)
+            color=self.color, pivot=self.pivot, length=self.barblength, zorder=2, **kwargs)
+
+
+@exporter.export
+class ArrowPlot(PlotVector, ValidationMixin):
+    """Make plots of wind barbs on a map with traits to refine the look of plotted elements."""
+
+    arrowscale = Union([Int(), Float(), Unicode()], allow_none=True, default_value=None)
+    arrowscale.__doc__ = """Number of data units per arrow length unit, e.g., m/s per plot
+    width; a smaller scale parameter makes the arrow longer. Default is `None`.
+
+    If `None`, a simple autoscaling algorithm is used, based on the average
+    vector length and the number of vectors. The arrow length unit is given by
+    the `key_length` attribute.
+
+    This trait corresponds to the keyword length in `matplotlib.pyplot.quiver`.
+    """
+
+    arrowkey = Tuple(Float(allow_none=True), Float(allow_none=True), Float(allow_none=True),
+                     Unicode(allow_none=True), Unicode(allow_none=True), default_value=None,
+                     allow_none=True)
+    arrowkey.__doc__ = """Set the characteristics of an arrow key using a tuple of values
+    representing (value, xloc, yloc, position, string).
+
+    Default is `None`.
+
+    If `None`, no vector key will be plotted.
+
+    value default is 100
+    xloc default is 0.85
+    yloc default is 1.02
+    position default is 'E' (options are 'N', 'S', 'E', 'W')
+    label default is an empty string
+
+    If you wish to change a characteristic of the arrowkey you'll need to have a tuple of five
+    elements, fill in the full tuple using `None` for those characteristics you wish to use the
+    default value and put in the new values for the other elements. This trait corresponds to
+    the keyword length in `matplotlib.pyplot.quiverkey`.
+    """
+
+    @observe('arrowscale', 'pivot', 'skip', 'earth_relative', 'color', 'arrowkey')
+    def _set_need_rebuild(self, _):
+        """Handle changes to attributes that need to regenerate everything."""
+        # Because matplotlib doesn't let you just change these properties, we need
+        # to trigger a clear and re-call of quiver()
+        self.clear()
+
+    def _build(self):
+        """Build the plot by calling needed plotting methods as necessary."""
+        x_like, y_like, u, v = self.plotdata
+
+        kwargs = plot_kwargs(u)
+
+        # Conditionally apply the proper transform
+        if 'transform' in kwargs and self.earth_relative:
+            kwargs['transform'] = ccrs.PlateCarree()
+
+        wind_slice = (slice(None, None, self.skip[0]), slice(None, None, self.skip[1]))
+
+        self.handle = self.parent.ax.quiver(
+            x_like[wind_slice], y_like[wind_slice],
+            u.values[wind_slice], v.values[wind_slice],
+            color=self.color, pivot=self.pivot, scale=self.arrowscale, **kwargs)
+
+        # The order here needs to match the order of the tuple
+        if self.arrowkey is not None:
+            key_kwargs = {'U': 100, 'X': 0.85, 'Y': 1.02, 'labelpos': 'E', 'label': ''}
+            for name, val in zip(key_kwargs, self.arrowkey):
+                if val is not None:
+                    key_kwargs[name] = val
+            self.parent.ax.quiverkey(self.handle, labelcolor=self.color, **key_kwargs)
 
 
 @exporter.export
@@ -2066,9 +2212,9 @@ class PlotGeometry(MetPyHasTraits):
         # largest Polygon or LineString from the collection. If MultiPoint, associate the label
         # with one of the Points in the MultiPoint, chosen based on the label hash.
         if isinstance(geo_obj, (MultiPolygon, MultiLineString)):
-            geo_obj = max(geo_obj, key=lambda x: x.length)
+            geo_obj = max(geo_obj.geoms, key=lambda x: x.length)
         elif isinstance(geo_obj, MultiPoint):
-            geo_obj = geo_obj[label_hash % len(geo_obj)]
+            geo_obj = geo_obj.geoms[label_hash % len(geo_obj.geoms)]
 
         # Get the list of coordinates of the polygon/line/point
         if isinstance(geo_obj, Polygon):
@@ -2145,7 +2291,7 @@ class PlotGeometry(MetPyHasTraits):
                 self.parent.ax.add_geometries([geo_obj], edgecolor=stroke,
                                               facecolor='none', crs=ccrs.PlateCarree())
             elif isinstance(geo_obj, MultiPoint):
-                for point in geo_obj:
+                for point in geo_obj.geoms:
                     lon, lat = point.coords[0]
                     self.parent.ax.plot(lon, lat, color=fill, marker=self.marker,
                                         transform=ccrs.PlateCarree())
