@@ -129,13 +129,16 @@ class SkewXAxis(maxis.XAxis):
         return SkewXTick(self.axes, None, major=major)
 
     # Needed to properly handle tight bbox
-    def _get_tick_bboxes(self, ticks, renderer):
+    def _get_ticklabel_bboxes(self, ticks, renderer):
         """Return lists of bboxes for ticks' label1's and label2's."""
         return ([tick.label1.get_window_extent(renderer)
                  for tick in ticks if tick.label1.get_visible() and tick.lower_in_bounds],
                 [tick.label2.get_window_extent(renderer)
                  for tick in ticks if tick.label2.get_visible() and tick.upper_in_bounds])
 
+    # Older name used on Matplotlib < 3.6
+    _get_tick_bboxes = _get_ticklabel_bboxes
+
     def get_view_interval(self):
         """Get the view interval."""
         return self.axes.upper_xlim[0], self.axes.lower_xlim[1]
@@ -279,7 +282,7 @@ class SkewT:
         rect : tuple[float, float, float, float], optional
             Rectangle (left, bottom, width, height) in which to place the axes. This
             allows the user to place the axes at an arbitrary point on the figure.
-        aspect : float, int, or 'auto', optional
+        aspect : float, int, or Literal['auto'], optional
             Aspect ratio (i.e. ratio of y-scale to x-scale) to maintain in the plot.
             Defaults to 80.5. Passing the string ``'auto'`` tells matplotlib to handle
             the aspect ratio automatically (this is not recommended for SkewT).
@@ -343,9 +346,9 @@ class SkewT:
 
         Parameters
         ----------
-        pressure : array_like
+        pressure : array-like
             pressure values
-        t : array_like
+        t : array-like
             temperature values, can also be used for things like dew point
         args
             Other positional arguments to pass to :func:`~matplotlib.pyplot.plot`
@@ -376,13 +379,13 @@ class SkewT:
 
         Parameters
         ----------
-        pressure : array_like
+        pressure : array-like
             pressure values
-        u : array_like
+        u : array-like
             U (East-West) component of wind
-        v : array_like
+        v : array-like
             V (North-South) component of wind
-        c:
+        c : array-like, optional
             An optional array used to map colors to the barbs
         xloc : float, optional
             Position for the barbs, in normalized axes coordinates, where 0.0
@@ -393,7 +396,7 @@ class SkewT:
         y_clip_radius : float, optional
             Space, in normalized axes coordinates, to leave above/below plot
             before clipping wind barbs in the y-direction. Defaults to 0.08.
-        plot_units: `pint.unit`
+        plot_units: `pint.Unit`
             Units to plot in (performing conversion if necessary). Defaults to given units.
         kwargs
             Other keyword arguments to pass to :func:`~matplotlib.pyplot.barbs`
@@ -448,11 +451,11 @@ class SkewT:
 
         Parameters
         ----------
-        t0 : array_like, optional
+        t0 : array-like, optional
             Starting temperature values in Kelvin. If none are given, they will be
             generated using the current temperature range at the bottom of
             the plot.
-        pressure : array_like, optional
+        pressure : array-like, optional
             Pressure values to be included in the dry adiabats. If not
             specified, they will be linearly distributed across the current
             plotted pressure range.
@@ -466,7 +469,7 @@ class SkewT:
 
         See Also
         --------
-        :func:`~metpy.calc.thermo.dry_lapse`
+        :func:`~metpy.calc.dry_lapse`
         :meth:`plot_moist_adiabats`
         :class:`matplotlib.collections.LineCollection`
 
@@ -507,11 +510,11 @@ class SkewT:
 
         Parameters
         ----------
-        t0 : array_like, optional
+        t0 : array-like, optional
             Starting temperature values in Kelvin. If none are given, they will be
             generated using the current temperature range at the bottom of
             the plot.
-        pressure : array_like, optional
+        pressure : array-like, optional
             Pressure values to be included in the moist adiabats. If not
             specified, they will be linearly distributed across the current
             plotted pressure range.
@@ -525,7 +528,7 @@ class SkewT:
 
         See Also
         --------
-        :func:`~metpy.calc.thermo.moist_lapse`
+        :func:`~metpy.calc.moist_lapse`
         :meth:`plot_dry_adiabats`
         :class:`matplotlib.collections.LineCollection`
 
@@ -565,10 +568,10 @@ class SkewT:
 
         Parameters
         ----------
-        mixing_ratio : array_like, optional
+        mixing_ratio : array-like, optional
             Unitless mixing ratio values to plot. If none are given, default
             values are used.
-        pressure : array_like, optional
+        pressure : array-like, optional
             Pressure values to be included in the isohumes. If not
             specified, they will be linearly distributed across the current
             plotted pressure range up to 600 mb.
@@ -592,7 +595,8 @@ class SkewT:
         # Default mixing level values if necessary
         if mixing_ratio is None:
             mixing_ratio = np.array([0.0004, 0.001, 0.002, 0.004, 0.007, 0.01,
-                                     0.016, 0.024, 0.032]).reshape(-1, 1)
+                                     0.016, 0.024, 0.032])
+        mixing_ratio = mixing_ratio.reshape(-1, 1)
 
         # Set pressure range if necessary
         if pressure is None:
@@ -618,13 +622,13 @@ class SkewT:
 
         Parameters
         ----------
-        y : array_like
+        y : array-like
             1-dimensional array of numeric y-values
-        x1 : array_like
+        x1 : array-like
             1-dimensional array of numeric x-values
-        x2 : array_like
+        x2 : array-like
             1-dimensional array of numeric x-values
-        which : string
+        which : str
             Specifies if `positive`, `negative`, or `both` areas are being shaded.
             Will be overridden by where.
         kwargs
@@ -637,7 +641,7 @@ class SkewT:
         See Also
         --------
         :class:`matplotlib.collections.PolyCollection`
-        :func:`matplotlib.axes.Axes.fill_betweenx`
+        :meth:`matplotlib.axes.Axes.fill_betweenx`
 
         """
         fill_properties = {'positive':
@@ -673,13 +677,11 @@ class SkewT:
 
         Parameters
         ----------
-        pressure : array_like
+        pressure : array-like
             Pressure values
-        t : array_like
+        t : array-like
             Temperature values
-        dewpoint : array_like
-            Dewpoint values
-        t_parcel : array_like
+        t_parcel : array-like
             Parcel path temperature values
         limit_shading : bool
             Eliminate shading below the LCL or above the EL, default is True
@@ -693,7 +695,7 @@ class SkewT:
         See Also
         --------
         :class:`matplotlib.collections.PolyCollection`
-        :func:`matplotlib.axes.Axes.fill_betweenx`
+        :meth:`matplotlib.axes.Axes.fill_betweenx`
 
         """
         return self.shade_area(pressure, t_parcel, t, which='positive', **kwargs)
@@ -707,13 +709,13 @@ class SkewT:
 
         Parameters
         ----------
-        pressure : array_like
+        pressure : array-like
             Pressure values
-        t : array_like
+        t : array-like
             Temperature values
-        t_parcel : array_like
+        t_parcel : array-like
             Parcel path temperature values
-        dewpoint : array_like
+        dewpoint : array-like
             Dew point values, optional
         kwargs
             Other keyword arguments to pass to :class:`matplotlib.collections.PolyCollection`
@@ -725,7 +727,7 @@ class SkewT:
         See Also
         --------
         :class:`matplotlib.collections.PolyCollection`
-        :func:`matplotlib.axes.Axes.fill_betweenx`
+        :meth:`matplotlib.axes.Axes.fill_betweenx`
 
         """
         if dewpoint is not None:
@@ -762,7 +764,7 @@ class Hodograph:
         ----------
         ax : `matplotlib.axes.Axes`, optional
             The `Axes` instance used for plotting
-        component_range : value
+        component_range : int
             The maximum range of the plot. Used to set plot bounds and control the maximum
             number of grid rings needed.
 
@@ -786,7 +788,7 @@ class Hodograph:
 
         Parameters
         ----------
-        increment : value, optional
+        increment : int, optional
             The value increment between rings
         kwargs
             Other kwargs to control appearance of lines
@@ -834,9 +836,9 @@ class Hodograph:
 
         Parameters
         ----------
-        u : array_like
+        u : array-like
             u-component of wind
-        v : array_like
+        v : array-like
             v-component of wind
         kwargs
             Other keyword arguments to pass to :meth:`matplotlib.axes.Axes.plot`
@@ -862,9 +864,9 @@ class Hodograph:
 
         Parameters
         ----------
-        u : array_like
+        u : array-like
             u-component of wind
-        v : array_like
+        v : array-like
             v-component of wind
         kwargs
             Other keyword arguments to pass to :meth:`matplotlib.axes.Axes.quiver`
@@ -895,11 +897,11 @@ class Hodograph:
 
         Parameters
         ----------
-        u : array_like
+        u : array-like
             u-component of wind
-        v : array_like
+        v : array-like
             v-component of wind
-        c : array_like
+        c : array-like
             data to use for colormapping (e.g. heights, pressure, wind speed)
         intervals: array-like, optional
             Array of intervals for c to use in coloring the hodograph.
