@@ -17,8 +17,8 @@ from metpy.calc import wind_speed
 from metpy.cbook import get_test_data
 from metpy.io import GiniFile
 from metpy.io.metar import parse_metar_file
-from metpy.plots import (BarbPlot, ContourPlot, FilledContourPlot, ImagePlot, MapPanel,
-                         PanelContainer, PlotGeometry, PlotObs)
+from metpy.plots import (ArrowPlot, BarbPlot, ContourPlot, FilledContourPlot, ImagePlot,
+                         MapPanel, PanelContainer, PlotGeometry, PlotObs, RasterPlot)
 from metpy.testing import needs_cartopy
 from metpy.units import units
 
@@ -90,7 +90,7 @@ def test_declarative_four_dims_error():
         pc.draw()
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.09)
 @needs_cartopy
 def test_declarative_contour():
     """Test making a contour plot."""
@@ -118,7 +118,38 @@ def test_declarative_contour():
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.01)
+@pytest.mark.mpl_image_compare(remove_text=False, tolerance=0.091)
+@needs_cartopy
+def test_declarative_titles():
+    """Test making a contour plot with multiple titles."""
+    data = xr.open_dataset(get_test_data('narr_example.nc', as_file_obj=False))
+
+    contour = ContourPlot()
+    contour.data = data
+    contour.field = 'Temperature'
+    contour.level = 700 * units.hPa
+    contour.contours = 30
+    contour.linewidth = 1
+    contour.linecolor = 'red'
+
+    panel = MapPanel()
+    panel.area = 'us'
+    panel.projection = 'lcc'
+    panel.layers = ['coastline']
+    panel.left_title = '700-hPa Temperature'
+    panel.right_title = 'Valid at a time'
+    panel.title = 'Plot of data'
+    panel.plots = [contour]
+
+    pc = PanelContainer()
+    pc.size = (8.0, 8)
+    pc.panels = [panel]
+    pc.draw()
+
+    return pc.figure
+
+
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.066)
 @needs_cartopy
 def test_declarative_smooth_contour():
     """Test making a contour plot using smooth_contour."""
@@ -218,7 +249,7 @@ def test_declarative_smooth_contour_order():
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.058)
 @needs_cartopy
 def test_declarative_figsize():
     """Test having an all float figsize."""
@@ -246,7 +277,7 @@ def test_declarative_figsize():
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.023)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.029)
 @needs_cartopy
 def test_declarative_smooth_field():
     """Test the smoothing of the field with smooth_field trait."""
@@ -275,7 +306,7 @@ def test_declarative_smooth_field():
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.011)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.708)
 @needs_cartopy
 def test_declarative_contour_cam():
     """Test making a contour plot with CAM data."""
@@ -332,7 +363,37 @@ def test_declarative_contour_options():
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.0188)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.08)
+@needs_cartopy
+def test_declarative_layers_plot_options():
+    """Test making a contour plot."""
+    data = xr.open_dataset(get_test_data('narr_example.nc', as_file_obj=False))
+
+    contour = ContourPlot()
+    contour.data = data
+    contour.field = 'Temperature'
+    contour.level = 700 * units.hPa
+    contour.contours = 5
+    contour.linewidth = 1
+    contour.linecolor = 'grey'
+
+    panel = MapPanel()
+    panel.area = 'us'
+    panel.projection = 'lcc'
+    panel.layers = ['coastline', 'usstates', 'borders']
+    panel.layers_edgecolor = ['blue', 'red', 'black']
+    panel.layers_linewidth = [0.75, 0.75, 1]
+    panel.plots = [contour]
+
+    pc = PanelContainer()
+    pc.size = (8, 8)
+    pc.panels = [panel]
+    pc.draw()
+
+    return pc.figure
+
+
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.0152)
 @needs_cartopy
 def test_declarative_contour_convert_units():
     """Test making a contour plot."""
@@ -345,7 +406,6 @@ def test_declarative_contour_convert_units():
     contour.contours = 30
     contour.linewidth = 1
     contour.linecolor = 'red'
-    contour.linestyle = 'dashed'
     contour.clabels = True
     contour.plot_units = 'degC'
 
@@ -363,7 +423,7 @@ def test_declarative_contour_convert_units():
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.246)
 @needs_cartopy
 def test_declarative_events():
     """Test that resetting traitlets properly propagates."""
@@ -407,6 +467,37 @@ def test_declarative_events():
     return pc.figure
 
 
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0)
+@needs_cartopy
+def test_declarative_raster_events():
+    """Test that resetting traitlets properly propagates in RasterPlot()."""
+    data = xr.open_dataset(get_test_data('narr_example.nc', as_file_obj=False))
+
+    raster = RasterPlot()
+    raster.data = data
+    raster.field = 'Temperature'
+    raster.level = 700 * units.hPa
+    raster.colormap = 'hot'
+
+    panel = MapPanel()
+    panel.area = 'us'
+    panel.projection = 'lcc'
+    panel.plots = [raster]
+
+    pc = PanelContainer()
+    pc.size = (8, 8.0)
+    pc.panels = [panel]
+    pc.draw()
+
+    # Update some properties to make sure it regenerates the figure
+    raster.level = 700 * units.hPa
+    raster.field = 'Geopotential_height'
+    raster.colormap = 'viridis'
+    raster.colorbar = 'vertical'
+
+    return pc.figure
+
+
 def test_no_field_error():
     """Make sure we get a useful error when the field is not set."""
     data = xr.open_dataset(get_test_data('narr_example.nc', as_file_obj=False))
@@ -474,6 +565,7 @@ def test_no_field_error_barbs():
         barbs.draw()
 
 
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.377)
 def test_projection_object(ccrs, cfeature):
     """Test that we can pass a custom map projection."""
     data = xr.open_dataset(get_test_data('narr_example.nc', as_file_obj=False))
@@ -523,7 +615,7 @@ def test_colorfill(cfeature):
 
 @pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.)
 def test_colorfill_horiz_colorbar(cfeature):
-    """Test that we can use ContourFillPlot."""
+    """Test that we can use ContourFillPlot with a horizontal colorbar."""
     data = xr.open_dataset(get_test_data('narr_example.nc', as_file_obj=False))
 
     contour = FilledContourPlot()
@@ -546,9 +638,9 @@ def test_colorfill_horiz_colorbar(cfeature):
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.005)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.0062)
 def test_colorfill_no_colorbar(cfeature):
-    """Test that we can use ContourFillPlot."""
+    """Test that we can use ContourFillPlot with no colorbar."""
     data = xr.open_dataset(get_test_data('narr_example.nc', as_file_obj=False))
 
     contour = FilledContourPlot()
@@ -624,7 +716,7 @@ def test_latlon():
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.37)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.292)
 @needs_cartopy
 def test_declarative_barb_options():
     """Test making a contour plot."""
@@ -642,7 +734,7 @@ def test_declarative_barb_options():
     panel = MapPanel()
     panel.area = 'us'
     panel.projection = 'data'
-    panel.layers = ['coastline', 'borders', 'usstates']
+    panel.layers = ['coastline', 'borders', 'usstates', 'land', 'ocean', 'lakes']
     panel.plots = [barb]
 
     pc = PanelContainer()
@@ -653,7 +745,98 @@ def test_declarative_barb_options():
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.612)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.37)
+@needs_cartopy
+def test_declarative_arrowplot():
+    """Test making a arrow plot."""
+    data = xr.open_dataset(get_test_data('narr_example.nc', as_file_obj=False))
+
+    arrows = ArrowPlot()
+    arrows.data = data
+    arrows.level = 300 * units.hPa
+    arrows.field = ['u_wind', 'v_wind']
+    arrows.skip = (10, 10)
+    arrows.color = 'blue'
+    arrows.pivot = 'mid'
+    arrows.arrowscale = 1000
+
+    panel = MapPanel()
+    panel.area = 'us'
+    panel.projection = 'data'
+    panel.layers = ['coastline', 'borders', 'usstates']
+    panel.plots = [arrows]
+
+    pc = PanelContainer()
+    pc.size = (8, 8)
+    pc.panels = [panel]
+    pc.draw()
+
+    return pc.figure
+
+
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.37)
+@needs_cartopy
+def test_declarative_arrowkey():
+    """Test making a arrow plot with an arrow key."""
+    data = xr.open_dataset(get_test_data('narr_example.nc', as_file_obj=False))
+
+    arrows = ArrowPlot()
+    arrows.data = data
+    arrows.level = 300 * units.hPa
+    arrows.field = ['u_wind', 'v_wind']
+    arrows.skip = (10, 10)
+    arrows.color = 'red'
+    arrows.pivot = 'mid'
+    arrows.arrowscale = 1e3
+    arrows.arrowkey = (100, None, 1.05, None, '100 kt')
+    arrows.plot_units = 'kt'
+
+    panel = MapPanel()
+    panel.area = 'us'
+    panel.projection = 'data'
+    panel.layers = ['coastline', 'borders', 'usstates']
+    panel.plots = [arrows]
+
+    pc = PanelContainer()
+    pc.size = (8, 8)
+    pc.panels = [panel]
+    pc.draw()
+
+    return pc.figure
+
+
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.37)
+@needs_cartopy
+def test_declarative_arrow_changes():
+    """Test making a arrow plot with an arrow key."""
+    data = xr.open_dataset(get_test_data('narr_example.nc', as_file_obj=False))
+
+    arrows = ArrowPlot()
+    arrows.data = data
+    arrows.level = 300 * units.hPa
+    arrows.field = ['u_wind', 'v_wind']
+    arrows.skip = (10, 10)
+    arrows.color = 'red'
+    arrows.pivot = 'mid'
+
+    panel = MapPanel()
+    panel.area = 'us'
+    panel.projection = 'data'
+    panel.layers = ['coastline', 'borders', 'usstates']
+    panel.plots = [arrows]
+
+    pc = PanelContainer()
+    pc.size = (8, 8)
+    pc.panels = [panel]
+    pc.draw()
+
+    arrows.color = 'green'
+    arrows.arrowkey = (None, 0.9, 1.1, 'W', '100 m/s')
+
+    return pc.figure
+
+
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.491)
 @needs_cartopy
 def test_declarative_barb_earth_relative():
     """Test making a contour plot."""
@@ -789,7 +972,7 @@ def test_declarative_global_gfs():
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.607)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=1.05)
 @needs_cartopy
 def test_declarative_barb_gfs():
     """Test making a contour plot."""
@@ -848,7 +1031,7 @@ def test_declarative_barb_scale():
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.466)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.667)
 @needs_cartopy
 def test_declarative_barb_gfs_knots():
     """Test making a contour plot."""
@@ -1117,7 +1300,7 @@ def test_declarative_sfc_obs_changes(ccrs):
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.00586)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.171)
 def test_declarative_colored_barbs(ccrs):
     """Test making a surface plot with a colored barb (gh-1274)."""
     data = pd.read_csv(get_test_data('SFC_obs.csv', as_file_obj=False),
@@ -1149,7 +1332,7 @@ def test_declarative_colored_barbs(ccrs):
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.00651)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.305)
 def test_declarative_sfc_obs_full(ccrs):
     """Test making a full surface observation plot."""
     data = pd.read_csv(get_test_data('SFC_obs.csv', as_file_obj=False),
@@ -1187,7 +1370,7 @@ def test_declarative_sfc_obs_full(ccrs):
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.16)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.355)
 @needs_cartopy
 def test_declarative_upa_obs():
     """Test making a full upperair observation plot."""
@@ -1224,10 +1407,10 @@ def test_declarative_upa_obs():
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.114)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.473)
 @needs_cartopy
 def test_declarative_upa_obs_convert_barb_units():
-    """Test making a full upperair observation plot."""
+    """Test making a full upperair observation plot with barbs converting units."""
     data = pd.read_csv(get_test_data('UPA_obs.csv', as_file_obj=False))
     data.units = ''
     data.units = {'pressure': 'hPa', 'height': 'meters', 'temperature': 'degC',
@@ -1522,6 +1705,32 @@ def test_declarative_contour_label_fontsize():
     return pc.figure
 
 
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0)
+@needs_cartopy
+def test_declarative_raster():
+    """Test making a raster plot."""
+    data = xr.open_dataset(get_test_data('narr_example.nc', as_file_obj=False))
+
+    raster = RasterPlot()
+    raster.data = data
+    raster.colormap = 'viridis'
+    raster.field = 'Temperature'
+    raster.level = 700 * units.hPa
+
+    panel = MapPanel()
+    panel.area = 'us'
+    panel.projection = 'lcc'
+    panel.layers = ['coastline']
+    panel.plots = [raster]
+
+    pc = PanelContainer()
+    pc.size = (8.0, 8)
+    pc.panels = [panel]
+    pc.draw()
+
+    return pc.figure
+
+
 @pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.607)
 @needs_cartopy
 def test_declarative_region_modifier_zoom_in():
@@ -1546,7 +1755,7 @@ def test_declarative_region_modifier_zoom_in():
     return pc.figure
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.02)
+@pytest.mark.mpl_image_compare(remove_text=True, tolerance=0.332)
 @needs_cartopy
 def test_declarative_region_modifier_zoom_out():
     """Test that '-' suffix on area string properly expands extent of map."""
@@ -1628,8 +1837,8 @@ def test_panel():
 def test_copy():
     """Test that the copy method works for all classes in `declarative.py`."""
     # Copies of plot objects
-    objects = [ImagePlot(), ContourPlot(), FilledContourPlot(), BarbPlot(), PlotObs(),
-               PlotGeometry()]
+    objects = [ImagePlot(), ContourPlot(), FilledContourPlot(), RasterPlot(), BarbPlot(),
+               PlotObs(), PlotGeometry()]
 
     for obj in objects:
         obj.time = datetime.now()
@@ -1653,7 +1862,7 @@ def test_copy():
     # Copies of plots in MapPanels should not point to same location in memory
     obj = MapPanel()
     obj.plots = [PlotObs(), PlotGeometry(), BarbPlot(), FilledContourPlot(), ContourPlot(),
-                 ImagePlot()]
+                 RasterPlot(), ImagePlot()]
     copied_obj = obj.copy()
 
     for i in range(len(obj.plots)):
@@ -1790,6 +1999,7 @@ def test_drop_traitlets_dir():
             BarbPlot,
             ContourPlot,
             FilledContourPlot,
+            RasterPlot,
             ImagePlot,
             MapPanel,
             PanelContainer,
