@@ -47,9 +47,9 @@ def test_skewt_api():
     return fig
 
 
-@pytest.mark.mpl_image_compare(remove_text=True, style='default', tolerance=0.)
+@pytest.mark.mpl_image_compare(remove_text=True, style='default', tolerance=0.32)
 def test_skewt_api_units():
-    """#Test the SkewT API when units are provided."""
+    """Test the SkewT API when units are provided."""
     with matplotlib.rc_context({'axes.autolimit_mode': 'data'}):
         fig = plt.figure(figsize=(9, 9))
         skew = SkewT(fig)
@@ -83,6 +83,20 @@ def test_skewt_default_aspect_empty():
     return fig
 
 
+@pytest.mark.mpl_image_compare(tolerance=0., remove_text=True, style='default')
+def test_skewt_mixing_line_args():
+    """Test plot_mixing_lines accepting kwargs for mixing ratio and pressure levels."""
+    # Explicitly pass default values as kwargs the, should recreate NWS SkewT PDF as above
+    fig = plt.figure(figsize=(12, 9))
+    skew = SkewT(fig, rotation=43)
+    mlines = np.array([0.0004, 0.001, 0.002, 0.004, 0.007, 0.01, 0.016, 0.024, 0.032])
+    press = units.Quantity(np.linspace(600, max(skew.ax.get_ylim())), 'mbar')
+    skew.plot_dry_adiabats()
+    skew.plot_moist_adiabats()
+    skew.plot_mixing_lines(mixing_ratio=mlines, pressure=press)
+    return fig
+
+
 @pytest.mark.mpl_image_compare(tolerance=0., remove_text=False, style='default',
                                savefig_kwargs={'bbox_inches': 'tight'})
 def test_skewt_tight_bbox():
@@ -130,7 +144,7 @@ def test_skewt_subplot_rect_conflict():
         SkewT(rect=(0.15, 0.35, 0.8, 0.3), subplot=(1, 1, 1))
 
 
-@pytest.mark.mpl_image_compare(tolerance=0., remove_text=True, style='default')
+@pytest.mark.mpl_image_compare(tolerance=0.0198, remove_text=True, style='default')
 def test_skewt_units():
     """Test that plotting with SkewT works with units properly."""
     fig = plt.figure(figsize=(9, 9))
@@ -350,7 +364,7 @@ def test_hodograph_plot_colormapped():
     return fig
 
 
-@pytest.mark.mpl_image_compare(tolerance=0, remove_text=True, style='default')
+@pytest.mark.mpl_image_compare(tolerance=0.141, remove_text=True, style='default')
 def test_skewt_barb_color():
     """Test plotting colored wind barbs on the Skew-T."""
     fig = plt.figure(figsize=(9, 9))
